@@ -4,7 +4,7 @@ use crate::explorer;
 use crate::input::Key;
 use crate::lua;
 use crate::permissions::Permissions;
-use crate::runner;
+use crate::runner::Runner;
 use crate::ui::Layout;
 use anyhow::{bail, Result};
 use chrono::{DateTime, Local};
@@ -1272,6 +1272,17 @@ pub enum ExternalMsg {
     /// Toggle mouse
     ToggleMouse,
 
+    /// Start piping the focused path to the given fifo path
+    ///
+    /// **Example:** `StartFifo: /tmp/xplr.fifo`
+    StartFifo(String),
+
+    /// Close the active fifo and stop piping.
+    StopFifo,
+
+    /// Toggle betwen {Start|Stop}Fifo
+    ToggleFifo(String),
+
     /// Log information message.
     ///
     /// **Example:** `LogInfo: launching satellite`
@@ -1363,6 +1374,9 @@ pub enum MsgOut {
     EnableMouse,
     DisableMouse,
     ToggleMouse,
+    StartFifo(String),
+    StopFifo,
+    ToggleFifo(String),
 }
 
 #[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
@@ -1507,17 +1521,19 @@ pub struct App {
 }
 
 impl App {
-    pub fn create(pwd: PathBuf, lua: &mlua::Lua) -> Result<Self> {
+    pub fn create(pwd: PathBuf, lua: &mlua::Lua, config_file: Option<PathBuf>) -> Result<Self> {
         let config = lua::init(lua)?;
 
-        let config_dir = dirs::config_dir()
-            .unwrap_or_else(|| PathBuf::from("."))
-            .join("xplr");
+        let config_file = if let Some(path) = config_file {
+            path
+        } else if let Some(dir) = dirs::home_dir() {
+            dir.join(".config").join("xplr").join("init.lua")
+        } else {
+            PathBuf::from("/").join("etc").join("xplr").join("init.lua")
+        };
 
-        let lua_script_file = config_dir.join("init.lua");
-
-        let (config, load_err) = if lua_script_file.exists() {
-            match lua::extend(lua, &lua_script_file.to_string_lossy().to_string()) {
+        let (config, load_err) = if config_file.exists() {
+            match lua::extend(lua, &config_file.to_string_lossy().to_string()) {
                 Ok(c) => (c, None),
                 Err(e) => (config, Some(e.to_string())),
             }
@@ -1595,6 +1611,8 @@ impl App {
             last_modes: Default::default(),
         };
 
+        fs::create_dir_all(app.session_path())?;
+
         if let Some(err) = load_err {
             app.log_error(err)
         } else {
@@ -1718,6 +1736,9 @@ impl App {
                 ExternalMsg::EnableMouse => self.enable_mouse(),
                 ExternalMsg::DisableMouse => self.disable_mouse(),
                 ExternalMsg::ToggleMouse => self.toggle_mouse(),
+                ExternalMsg::StartFifo(f) => self.start_fifo(f),
+                ExternalMsg::StopFifo => self.stop_fifo(),
+                ExternalMsg::ToggleFifo(f) => self.toggle_fifo(f),
                 ExternalMsg::LogInfo(l) => self.log_info(l),
                 ExternalMsg::LogSuccess(l) => self.log_success(l),
                 ExternalMsg::LogWarning(l) => self.log_warning(l),
@@ -1756,12 +1777,16 @@ impl App {
             })
             .or_else(|| default.map(|a| a.messages().clone()))
             .unwrap_or_else(|| {
-                vec![
-                    ExternalMsg::SwitchModeBuiltin("recover".into()),
-                    ExternalMsg::LogWarning(
-                        "Key map not found. Let's calm down, escape, and try again.".into(),
-                    ),
-                ]
+                if self.config().general().disable_recover_mode() {
+                    vec![ExternalMsg::LogWarning("Key map not found.".into())]
+                } else {
+                    vec![
+                        ExternalMsg::SwitchModeBuiltin("recover".into()),
+                        ExternalMsg::LogWarning(
+                            "Key map not found. Let's calm down, escape, and try again.".into(),
+                        ),
+                    ]
+                }
             });
 
         for msg in msgs {
@@ -2448,6 +2473,21 @@ impl App {
         Ok(self)
     }
 
+    fn start_fifo(mut self, path: String) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::StartFifo(path));
+        Ok(self)
+    }
+
+    fn stop_fifo(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::StopFifo);
+        Ok(self)
+    }
+
+    fn toggle_fifo(mut self, path: String) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::ToggleFifo(path));
+        Ok(self)
+    }
+
     pub fn log_info(mut self, message: String) -> Result<Self> {
         self.logs_hidden = false;
         self.logs.push(Log::new(LogLevel::Info, message));
@@ -2796,24 +2836,8 @@ impl App {
             last_modes: self.last_modes.clone(),
         }
     }
-
-    pub fn run(self, focused_path: Option<PathBuf>, lua: &mlua::Lua) -> Result<Option<String>> {
-        runner::run(self, focused_path, lua)
-    }
 }
 
-/// Run xplr TUI
-pub fn run(
-    pwd: PathBuf,
-    focused_path: Option<PathBuf>,
-    on_load: Option<Vec<ExternalMsg>>,
-) -> Result<Option<String>> {
-    let lua = mlua::Lua::new();
-    let mut app = App::create(pwd, &lua)?;
-    if let Some(msgs) = on_load {
-        for msg in msgs {
-            app = app.enqueue(Task::new(MsgIn::External(msg), None));
-        }
-    }
-    app.run(focused_path, &lua)
+pub fn runner(path: Option<PathBuf>) -> Result<Runner> {
+    Runner::new(path)
 }
